import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.event.MouseInputListener;
import javax.imageio.ImageIO;
import java.io.File;
import java.util.*;
import javax.swing.BorderFactory;
import javax.swing.border.*;
import java.awt.Font;

public class SquiggleFrame extends JFrame
{
    Squiggle sq;
    JPanel SquigglePanel;
    //mode 0 = picture grid, mode 1 = all panels, mode 2 = full screen scribble
    int mode;
    
    //SQUIGGLE
    public void setSquiggle(Squiggle s) {
        sq = s;
    }
    
    public Squiggle getSquiggle() {
        return sq;
    }
    
    //TAKE CARE OF MODES
    public void setMode(int m) {
        mode = m;
        //remove all components
        this.getContentPane().removeAll();
        if (mode == 0) {
            this.getContentPane().add(sq.getTilePanel());
            this.getContentPane().add(sq.getControlPanel(), "North");
        }
        else if (mode == 2) {
            SquigglePanel = new JPanel(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            SquigglePanel.add(sq, c);
            this.getContentPane().add(SquigglePanel);
            this.getContentPane().add(sq.getControlPanel(), "North");
        }
        else {
            this.getContentPane().add(sq.getMapPanel(), "East");
            this.getContentPane().add(sq.getControlPanel(), "North");
            SquigglePanel = new JPanel(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            SquigglePanel.add(sq, c);
            this.getContentPane().add(SquigglePanel);
        }
        this.validate();
        this.setVisible(true);
    }
    
    public void repaintSquiggle() {
        this.getContentPane().remove(2);
        SquigglePanel = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        SquigglePanel.add(sq, c);
        this.getContentPane().add(SquigglePanel);
        validate();
        repaint();
    }
    
    public int getMode() {
        return mode;
    }
    
    //MAIN METHOD 2
    public static SquiggleFrame init(String[] args)
    {
        //create frame
        SquiggleFrame f = new SquiggleFrame();
        //get arguments
        int w = Integer.parseInt(args[args.length-2]);
        int h = Integer.parseInt(args[args.length-1]);
        String[] s = new String[args.length-2];
        System.arraycopy(args, 0, s, 0, args.length-2);
        //initialize squiggle and components
        Squiggle squiggle = new Squiggle(s, f, w, h);
        DrawingListener listener = new DrawingListener(squiggle);
        squiggle.addMouseListener(listener);
        squiggle.addMouseMotionListener(listener);
        f.setSquiggle(squiggle);
        //initialize frame
        f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        f.getContentPane().add(squiggle.getTilePanel());
        f.setSize(400,400);
        f.setLocation(200,200);
        f.setExtendedState(Frame.MAXIMIZED_BOTH);
        f.setVisible(true);
        return f;
    }
    
    //MAIN METHOD
    public static void main(String[] args)
    {
        String[] s = {"winter.jpg", "sunset.jpg", "lily.jpg", "hills.jpg", "200", "150"};
        SquiggleFrame f = init(s);
        Squiggle sq = f.getSquiggle();
        //String[] t = {"sunset.jpg", "sunset.jpg", "winter.jpg", "winter.jpg", "img5.jpg","img4.jpg","img3.jpg","img2.jpg","img1.jpg"};
        //String[] u = {"img1.jpg","img2.jpg","img3.jpg","img2.jpg","img1.jpg", "hills.jpg", "hills.jpg", "lily.jpg", "lily.jpg"};
        //sq.setSegmentations(t);
        //sq.setConfusionMaps(u);
        //sq.setGroundImages(t);
        //sq.setExperiment(1);
    }
}

class Squiggle extends JPanel
{
    SquiggleFrame sqframe;
    BufferedImage image;
    String imagename;
    BufferedImage backimage;
    Color color;
    Stroke stroke;
    Vector<int[]> currentsq = new Vector<int[]>();
    Vector<Vector<int[]>> squiggles_temp = new Vector<Vector<int[]>>();
    Vector<Vector<int[]>> squiggles = new Vector<Vector<int[]>>();
    Vector<String> imagenames = new Vector<String>();
    Vector<String[]> association = new Vector<String[]>();
    //images/segmentations/confusion maps
    String[] actualimages = {"uninitialized"};
    String[] segmentations = {"uninitialized"};
    String[] confusionmaps = {"uninitialized"};
    String[] groundimages = {"uninitialized"};
    //labels for images
    JLabel actuallab = new JLabel("Original Images");
    JLabel seglab = new JLabel("Segmentations");
    JLabel conflab = new JLabel();
    JLabel groundlab = new JLabel("Foreground");
    int mapW = 150; int mapH = 150;
    int tileW = 180; int tileH = 130;
    int imgW = 500; int imgH = 500;
    int buttonwidth = 130; int buttonheight = 40;
    boolean isDone;
    boolean firsttime;
    boolean nextGroup;
    JLabel current_color = new JLabel();
    //variables for pixels
    int maxpixels = 20000;
    public int maxpixels_per_itr = 10000;
    int imagepixels;
    int totalpixels;
    public int pixels_since_last;
    //variable for score
    float score = 0.00f;
    //variable for iteration
    int iternum;
    //jlabels for pixels and score
    JLabel pixelleftlab = new JLabel();
    JLabel iterationlab = new JLabel();
    JLabel scorelab = new JLabel();
    JLabel statuslabel = new JLabel();
    //checking for bad squiggles
    boolean badsquiggle;
    int experiment;
    //color panels
    ColorPanel foregroundp;
    ColorPanel backgroundp;
    
    //CONSTRUCTORS
    public Squiggle()
    {
        color = Color.red;
        stroke = new BasicStroke(3f, BasicStroke.CAP_BUTT,
        BasicStroke.JOIN_MITER);
        isDone = false;
        firsttime = true;
        nextGroup = false;
        badsquiggle = false;
        imagepixels = 0;
        totalpixels = 0;
        pixels_since_last = 0;
        refreshPixelCount();
        setScore(0);
        experiment = 0;
        iternum = 1;
    }
    
    public Squiggle(String[] acim, SquiggleFrame sqfr) {
        this();
        sqframe = sqfr;
        String[] actualtemp = new String[acim.length / 2];
        String[] groundtemp = new String[acim.length / 2];
        System.arraycopy(acim, 0, actualtemp, 0, acim.length / 2);
        System.arraycopy(acim, acim.length/2, groundtemp, 0, acim.length/2);
        setActualImages(actualtemp);
        setGroundImages(groundtemp);
    }
    
    public Squiggle(String[] acim, SquiggleFrame sqfr, int wid, int hei) {
        this(acim, sqfr);
        tileW = wid;
        tileH = hei;
    }
    
    //PAINT COMPONENT
    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        if(image == null)
            initImage();
        g.drawImage(image, 0, 0, this);
    }
    
    //DRAW LINE
    public void draw(Point start, Point end)
    {
        Graphics2D g2 = image.createGraphics();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setPaint(color);
        g2.setStroke(stroke);
        g2.draw(new Line2D.Double(start, end));
        g2.dispose();
        repaint();
    }
    //DRAW WITH COLOR
    public void draw(Point start, Point end, Color col)
    {
        Graphics2D g2 = image.createGraphics();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setPaint(col);
        g2.setStroke(stroke);
        g2.draw(new Line2D.Double(start, end));
        g2.dispose();
        repaint();
    }
    
    //CLEAR IMAGE AND RETAIN BACK IMAGE
    private void clearImage()
    {
        Graphics g = image.getGraphics();
        g.setColor(getBackground());
        g.fillRect(0, 0, image.getWidth(), image.getHeight());
        g.drawImage(backimage, 0, 0, null);
        g.dispose();
        repaint();
    }
    
    //INITIALIZE IMAGE
    private void initImage()
    {
        firsttime = false;
        int w = backimage.getWidth();
        int h = backimage.getHeight();
        image = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
        //manipulate graphic
        Graphics2D g2 = image.createGraphics();
        g2.setPaint(getBackground());
        g2.fillRect(0,0,w,h);
        g2.drawImage(backimage, 0, 0, null);
        g2.dispose();
    }
    
    //GET IMAGE
    public String getImagen() {
        return getAssociation(imagename);
    }
    
    public int getImagenum() {
        String s = getAssociation(imagename);
        for (int i = 0 ; i < actualimages.length ; i++)
        {
            if(actualimages[i].equals(s))
                return i++;
        }
        return -1;
    }
    
    //SET IMAGE WITHOUT CLEARING
    public void setImage2(String filename) {
        backimage = null;
        try {
            backimage = ImageIO.read(new File(filename));
            imagename = filename.toLowerCase();
        } catch (Exception e) {
            e.printStackTrace();
        }
        backimage = resizeImage(backimage, imgW, imgH);
        this.setPreferredSize(new Dimension(backimage.getWidth(), backimage.getHeight()));
        //refresh image pixels
        refreshImagePixels();
    }
    
    //SET IMAGE WITH CLEARING
    public void setImage(String filename) {
        backimage = null;
        try {
            backimage = ImageIO.read(new File(filename));
            imagename = filename.toLowerCase();
        } catch (Exception e) {
            e.printStackTrace();
        }
        backimage = resizeImage(backimage, imgW, imgH);
        this.setPreferredSize(new Dimension(backimage.getWidth(), backimage.getHeight()));
        initImage();
        showSquiggle();
        repaint();
        if (sqframe.getMode() == 1) sqframe.repaintSquiggle();
        //refresh image pixels
        refreshImagePixels();
    }
    
    //RENDER IMAGE
    public void renderImage(int[][][] pixels) {
        for (int x = 0; x < pixels.length; x++) {
            for (int y = 0; y < pixels[0].length; y++) {
                int[] rgb = pixels[x][y];
                backimage.setRGB(y, x, toRGB(rgb));
            }
        }
        clearImage();
    }
    
    //SET IMAGE SIZE
    public void setImageSize(int w, int h) {
        imgW = w-1;
        imgH = h-1;
    }
    
    public int[] getImageSize() {
        int[] temp = new int[2];
        temp[0] = backimage.getWidth()+1; temp[1] = backimage.getHeight()+1;
        return temp;
    }
    
    //RESIZE IMAGE
    public BufferedImage resizeImage(BufferedImage tempimage, int tempW, int tempH) {
        int currentW = tempimage.getWidth(); int currentH = tempimage.getHeight();
        //if (currentW <= tempW && currentH <= tempH) return tempimage;
        int newW = 0; int newH = 0;
        //check which side to scale by
        if ( tempW*currentH > tempH*currentW ) {
            //scale by height
            newH = tempH;
            newW = currentW * tempH / currentH;
        }
        else {
            //scale by width
            newW = tempW;
            newH = currentH * tempW / currentW;
        }
        newW--; newH--;
        BufferedImage resultimage = new BufferedImage(newW, newH, tempimage.getType());
        Graphics2D g = resultimage.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.drawImage(tempimage, 0, 0, newW, newH, 0, 0, tempimage.getWidth(), tempimage.getHeight(), null);
        g.dispose();
        return resultimage;
    }
    
    //IMAGE PIXEL STUFF
    public void setMaxPixels(int max) {
        maxpixels = max;
    }
    
    public int getMaxPixels() {
        return maxpixels;
    }
    
    public int getImagePixels() {
        return imagepixels;
    }
    
    public int getTotalPixels() {
        return totalpixels;
    }
    
    public void refreshPixelCount() {
        pixelleftlab.setText("Pixels Left to be Drawn: " + (maxpixels_per_itr - pixels_since_last));
        iterationlab.setText("Iteration Number: " + iternum + "/10");
    }
    
    public void setIterNum(int n) {
        iternum = n;
        refreshPixelCount();
    }
    
    public void resetPixelCount() {
        imagepixels = 0;
        totalpixels = 0;
        pixels_since_last = 0;
    }
    
    public void refreshImagePixels() {
        imagepixels = 0;
        String filename = getAssociation(imagename);
        for (int i = 0; i < imagenames.size(); i++) {
            if (imagenames.get(i).equals(filename)) {
                imagepixels += squiggles.get(i).size()-1;
            }
        }
        refreshPixelCount();
    }
    
    public void refreshAllPixels() {
        totalpixels = 0;
        for (int i = 0; i < squiggles.size(); i++) {
            totalpixels += squiggles.get(i).size()-1;
        }
        pixels_since_last = 0;
        for (int i = 0; i < squiggles_temp.size(); i++) {
            pixels_since_last += squiggles_temp.get(i).size()-1;
        }
        refreshImagePixels();
    }
    
    //SCORE STUFF
    public void setScore(float sc) {
        score = sc;
        scorelab.setText("Your Score: " + score + "%");
    }
    
    public float getScore() {
        return score;
    }
    
    //STATS STUFF
    public void setStatus(String s) {
        statuslabel.setText(s);
    }
    
    public String getStatus() {
        return statuslabel.getText();
    }
    
    //SETTING IMAGE STRING ARRAYS
    public void setActualImages(String[] s) {
        actualimages = s;
    }
    
    public void setSegmentations(String[] s) {
        segmentations = s;
    }
    
    public void setConfusionMaps(String[] s) {
        confusionmaps = s;
    }
    
    public void setGroundImages(String[] s) {
        groundimages = s;
    }
    
    public String[] getActualImages() {
        return actualimages;
    }
    
    public String[] getSegmentations() {
        return segmentations;
    }
    
    public String[] getConfusionMaps() {
        return confusionmaps;
    }
    
    public String[] getGroundImages() {
        return groundimages;
    }
    
    //SETTING LABELS
    public void setActualLabel(String s) {
        actuallab.setText(s);
    }
    
    public void setSegmentationLabel(String s) {
        seglab.setText(s);
    }
    public void setConfusionMapLabel(String s) {
        conflab.setText(s);
    }
    public void setGroundLabel(String s) {
        groundlab.setText(s);
    }
    
    //SETTING WIDTH/HEIGHT VALUES
    public void setMapWH(int w, int h) {
        mapW = w;
        mapH = h;
    }
    public void setTileWH(int w, int h) {
        tileW = w;
        tileH = h;
    }
    public void setButtonWH(int w, int h) {
        buttonwidth = w;
        buttonheight = h;
    }
    public int[] getMapWH() {
        int[] temp = new int[2];
        temp[0] = mapW;
        temp[1] = mapH;
        return temp;
    }
    public int[] getTileWH() {
        int[] temp = new int[2];
        temp[0] = tileW;
        temp[1] = tileH;
        return temp;
    }
    public int[] getButtonWH() {
        int[] temp = new int[2];
        temp[0] = buttonwidth;
        temp[1] = buttonheight;
        return temp;
    }
    //DONE BUTTON
    public void setDone(boolean done_temp) {
        isDone = done_temp;
    }
    
    public boolean getDone() {
        return isDone;
    }
    
    //NEXT GROUP BUTTON
    public boolean getNextGroup() {
        return nextGroup;
    }
    
    public void setNextGroup(boolean next_temp) {
        nextGroup = next_temp;
    }
    
    //EXPERIMENT
    public void setExperiment(int exp) {
        experiment = exp;
    }
    
    public int getExperiment() {
        return experiment;
    }
    
    //DEALING WITH ARGB VALUES
    public int[] fromARGB(int argb) {
        int a = (argb >>> 24) & 0xff;
        int r = (argb >>> 16) & 0xff;
        int g = (argb >>> 8) & 0xff;
        int b = argb & 0xff;
        int[] result = new int[4];
        result[0]=a; result[1]=r; result[2]=g; result[3]=b;
        return result;
    }
    
    public int toARGB(int[] argb) {
        int a = argb[0]; int r = argb[1]; int g = argb[2]; int b = argb[3];
        int result = ((a << 24) & 0xff000000) + ((r << 16) & 0xff0000) + ((g << 8) & 0xff00) + (b & 0xff);
        return result;
    }
    
    public int toRGB(int[] argb) {
        int a = 0; int r = argb[0]; int g = argb[1]; int b = argb[2];
        int result = ((a << 24) & 0xff000000) + ((r << 16) & 0xff0000) + ((g << 8) & 0xff00) + (b & 0xff);
        return result;
    }
    
    //DEALING WITH SQUIGGLES
    public void addPoint(int x, int y) {
        //check for out of bounds squiggle
        if ((x > backimage.getWidth() || x < 0 || y > backimage.getHeight() || y < 0) && badsquiggle == false) {
            finishSquiggle();
            badsquiggle = true;
        }
//         if (pixels_since_last == maxpixels_per_itr) {
//             finishSquiggle();
//             badsquiggle = true;
//         }
        int[] temp = new int[2];
        temp[0] = x; temp[1] = y;
        currentsq.add(temp);
        //calculate pixels drawn
        if (badsquiggle == false) {
            totalpixels++;
            imagepixels++;
            pixels_since_last++;
            refreshPixelCount();
            if (totalpixels == maxpixels || pixels_since_last == maxpixels_per_itr) finishSquiggle();
            
        }
    }
    
    public void finishSquiggle() {
        if (badsquiggle == true) {
            badsquiggle = false;
            currentsq.clear();
            clearImage();
            showSquiggle();
            refreshAllPixels();
        }
        else {
            //add color
            int[] color_temp = {0, -1};
            if (color == Color.red) {
                currentsq.add(color_temp);
            }
            else {
                color_temp[0] = 1;
                currentsq.add(color_temp);
            }
            //add to squiggles
            squiggles_temp.add((Vector<int[]>)currentsq.clone());
            squiggles.add((Vector<int[]>)currentsq.clone());
            imagenames.add(getAssociation(this.getImagen()));
            currentsq.clear();
        }
    }
    
    public Vector<Vector<int[]>> getSquiggle() {
        Vector<Vector<int[]>> result = (Vector<Vector<int[]>>)squiggles_temp.clone();
        squiggles_temp.clear();
        return result;
    }
    
    public Vector<Vector<int[]>> getSquiggle(String s) {
        String filename = getAssociation(s);
        Vector<Vector<int[]>> result = new Vector<Vector<int[]>>();
        for (int i = 0; i < imagenames.size(); i++) {
            if (imagenames.get(i).equals(filename)) {
                result.add(squiggles.get(i));
            }
        }
        return result;
    }
    
    public Vector<Vector<int[]>> getAllSquiggles() {
        Vector<Vector<int[]>> result = (Vector<Vector<int[]>>)squiggles.clone();
        return result;
    }
    
    public Vector<String> getAllSquigglesNames() {
        Vector<String> result = (Vector<String>)imagenames.clone();
        return result;
    }
    
    public void resetAllSquiggles() {
        iternum = 1;
        squiggles.clear();
        imagenames.clear();
        currentsq.clear();
        refreshAllPixels();
        clearImage();
        setImage(getImagen());
    }
    
    public boolean getBadSquiggle() {
        return badsquiggle;
    }
    
    //show the squiggle on the image
    public void showSquiggle() {
        String s = getAssociation(this.getImagen());
        for (int i = 0; i < imagenames.size(); i++) {
            if (imagenames.get(i).equals(s)) {
                Vector<int[]> temp = squiggles.get(i);
                Color col = Color.red;
                if ((temp.get(temp.size()-1))[0] == 1) col = Color.blue;
                for (int j = 0; j < temp.size()-2; j++) {
                    int[] coord = temp.get(j);
                    int[] coord2 = temp.get(j+1);
                    Point start = new Point(coord[0], coord[1]);
                    Point end = new Point(coord2[0], coord2[1]);
                    this.draw(start, end, col);
                }
            }
        }
    }
    
//     //ASSOCIATING IMAGE - SEGMENTATION - CONFUSION MAP
//     public void associate(String s1, String s2, String s3) {
//         for (int i = 0; i < association.size(); i++) {
//             String[] temp2 = association.get(i);
//             if (temp2[0].equals(s1)) {
//                 association.remove(i);
//                 i--;
//             }
//         }
//         String[] temp = new String[3];
//         temp[0] = s1.toLowerCase(); temp[1] = s2.toLowerCase(); temp[2] = s3.toLowerCase();
//         association.add(temp);
//     }

    //ASSOCIATING IMAGE - SEGMENTATION - CONFUSION MAP
    public void associate(String s1, String s2) {
        for (int i = 0; i < association.size(); i++) {
            String[] temp2 = association.get(i);
            if (temp2[0].equals(s1)) {
                association.remove(i);
                i--;
            }
        }
        String[] temp = new String[2];
        temp[0] = s1.toLowerCase(); temp[1] = s2.toLowerCase();
        association.add(temp);
    }
    
    public void associate(String s1, String s2, String s3, String s4) {
        for (int i = 0; i < association.size(); i++) {
            String[] temp2 = association.get(i);
            if (temp2[0].equals(s1)) {
                association.remove(i);
                i--;
            }
        }
        String[] temp = new String[4];
        temp[0] = s1.toLowerCase(); temp[1] = s2.toLowerCase(); temp[2] = s3.toLowerCase(); temp[3] = s4.toLowerCase();
        association.add(temp);
    }
    
    public String getAssociation(String s) {
        if (experiment == 0) {
            for (int i = 0; i < association.size(); i++) {
                String[] temp = association.get(i);
                if (temp[0].equals(s)) return temp[0];
                else if (temp[1].equals(s)) return temp[0]; // || temp[2].equals(s)
            }
            return s;
        }
        else if (experiment == 1) {
            for (int i = 0; i < association.size(); i++) {
                String[] temp = association.get(i);
                if (temp[0].equals(s)) return temp[0];
                else if (temp[1].equals(s) || temp[2].equals(s) || temp[3].equals(s)) return temp[0];
            }
            return s;
        }
        else {
            for (int i = 0; i < association.size(); i++) {
                String[] temp = association.get(i);
                if (temp[0].equals(s)) return temp[0];
                else if (temp[1].equals(s) || temp[2].equals(s)) return temp[0];  
            }
            return s;
        }    }
    
    //SWAP COLORS
    public void swapColor(Color col) {
        if (col.equals(Color.red)) {
            backgroundp.setBackground(Color.red);
            foregroundp.setBackground(Color.white);
        }
        else {
            foregroundp.setBackground(Color.blue);
            backgroundp.setBackground(Color.white);
        }
    }
        
    
    //INITIALIZE SELECTION PANEL
    public JScrollPane getSelectionPanel()
    {
        String[] images;
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(0,2,5,5));
        JPanel panel2 = new JPanel();
        panel2.setLayout(new GridLayout(0,2,5,5));
        images = new String[actualimages.length * 2];
        for (int i = 0; i < actualimages.length; i++) {
            images[i*2] = actualimages[i];
            images[i*2+1] = segmentations[i];
            //images[i*3+1] = segmentations[i];
            //images[i*2+1] = groundimages[i];
        }
        //add labels
        JPanel temppanel = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        temppanel.add(actuallab, c);
        panel2.add(temppanel);
//         temppanel = new JPanel(new GridBagLayout());
//         temppanel.add(groundlab, c);
//         panel2.add(temppanel);
        temppanel = new JPanel(new GridBagLayout());
        temppanel.add(seglab, c);
        panel2.add(temppanel);        //set font sizes
        Font curFont = actuallab.getFont();
        int fontsize = 20;
        actuallab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        seglab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        conflab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        groundlab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        for(int j = 0; j < images.length; j++)
        {
            //create action listener
            MouseListener l = new MouseListener()
            {
                public void mousePressed(MouseEvent e)
                {
                    ImagePanel impanel = (ImagePanel)e.getSource();
                    setImage(impanel.getImageName());
                }
                public void mouseEntered(MouseEvent e) {}
                public void mouseExited(MouseEvent e) {}
                public void mouseClicked(MouseEvent e) {}
                public void mouseReleased(MouseEvent e) {}
            };
            //create image panel
            BufferedImage tempimage = null;
            try {
                tempimage = ImageIO.read(new File(images[j]));
            } catch (Exception e) {
                e.printStackTrace();
            }
            int newW = mapW;
            int newH = mapH;
            BufferedImage tempimage2 = resizeImage(tempimage, newW, newH);
            ImagePanel impanel = new ImagePanel(tempimage2, images[j], newW, newH);
            if (experiment != 3) impanel.addMouseListener(l);
            panel.add(impanel);
        }
        JPanel panel3 = new JPanel();
        panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));
        panel3.add(panel2);
        panel3.add(panel);
        JScrollPane scrollPane = new JScrollPane(panel3);
        scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        return scrollPane;
    }
    
    //INITIALIZE MAPS PANEL
    public JScrollPane getMapPanel()
    {
        //for images
        JPanel panel = new JPanel();
        //for labels
        JPanel panel2 = new JPanel();
        if (experiment == 0) {
            panel.setLayout(new GridLayout(0,2,5,5));
            panel2.setLayout(new GridLayout(0,2,5,5));
        }
        else if (experiment == 1) {
            panel.setLayout(new GridLayout(0,4,5,5));
            panel2.setLayout(new GridLayout(0,4,5,5));
        }
        else {
            panel.setLayout(new GridLayout(0,3,5,5));
            panel2.setLayout(new GridLayout(0,3,5,5));
        }
        //build up images array
        String[] images;
        if (experiment == 0) {
            if (confusionmaps[0].equals("uninitialized")) return getSelectionPanel(); //segmentations[0].equals("uninitialized") && 
            else {
                images = new String[actualimages.length * 2];
                for (int i = 0; i < actualimages.length; i++) {
                    images[i*2] = actualimages[i];
                    images[i*2+1] = segmentations[i];
                    //images[i*3+1] = confusionmaps[i];
                }
            }
        }
        else if (experiment == 1) {
            if (segmentations[0].equals("uninitialized") || confusionmaps[0].equals("uninitialized") || groundimages[0].equals("uninitialized")) return getSelectionPanel();
            else {
                images = new String[actualimages.length * 4];
                for (int i = 0; i < actualimages.length; i++) {
                    images[i*4] = actualimages[i];
                    images[i*4+1] = segmentations[i];
                    images[i*4+2] = confusionmaps[i];
                    images[i*4+3] = groundimages[i];
                }
            }
        }
        else {
            if (segmentations[0].equals("uninitialized") || groundimages[0].equals("uninitialized")) return getSelectionPanel();  
            else {
                images = new String[actualimages.length * 3];
                for (int i = 0; i < actualimages.length; i++) {
                    images[i*3] = actualimages[i];
                    images[i*3+1] = segmentations[i];
                    images[i*3+2] = groundimages[i];
                }
            }
        }        //add labels
        if (experiment == 0) {
            JPanel temppanel = new JPanel(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            temppanel.add(actuallab, c);
            panel2.add(temppanel);
            temppanel = new JPanel(new GridBagLayout());
            temppanel.add(seglab, c);
            panel2.add(temppanel);
            //temppanel = new JPanel(new GridBagLayout());
            //temppanel.add(conflab, c);
            //panel2.add(temppanel);
        }
        else if (experiment == 1) {
            JPanel temppanel = new JPanel(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            temppanel.add(actuallab, c);
            panel2.add(temppanel);
            temppanel = new JPanel(new GridBagLayout());
            temppanel.add(seglab, c);
            panel2.add(temppanel);
            temppanel = new JPanel(new GridBagLayout());
            temppanel.add(conflab, c);
            panel2.add(temppanel);
            temppanel = new JPanel(new GridBagLayout());
            temppanel.add(groundlab, c);
            panel2.add(temppanel);
        }
        else {
            JPanel temppanel = new JPanel(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            temppanel.add(actuallab, c);
            panel2.add(temppanel);
            temppanel = new JPanel(new GridBagLayout());
            temppanel.add(seglab, c);
            panel2.add(temppanel);
            temppanel = new JPanel(new GridBagLayout());
            temppanel.add(groundlab, c);
            panel2.add(temppanel);
        }
        //set font sizes
        Font curFont = actuallab.getFont();
        int fontsize = 20;
        actuallab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        seglab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        conflab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        groundlab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        for(int j = 0; j < images.length; j++)
        {
            //create action listener
            MouseListener l = new MouseListener()
            {
                public void mousePressed(MouseEvent e)
                {
                    ImagePanel impanel = (ImagePanel)e.getSource();
                    if (experiment != 3) setImage(impanel.getImageName());
                    else {repaint(); revalidate();}
                }
                public void mouseEntered(MouseEvent e) {}
                public void mouseExited(MouseEvent e) {}
                public void mouseClicked(MouseEvent e) {}
                public void mouseReleased(MouseEvent e) {}
            };
            //create image panel
            BufferedImage tempimage = null;
            try {
                tempimage = ImageIO.read(new File(images[j]));
            } catch (Exception e) {
                e.printStackTrace();
            }
            int newW = mapW;
            int newH = mapH;
            BufferedImage tempimage2 = resizeImage(tempimage, newW, newH);
            ImagePanel impanel = new ImagePanel(tempimage2, images[j], newW, newH);
            impanel.addMouseListener(l);
            panel.add(impanel);
        }
        JPanel panel3 = new JPanel();
        panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));
        panel3.add(panel2);
        panel3.add(panel);
        JScrollPane scrollPane = new JScrollPane(panel3);
        scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        return scrollPane;
    }
    
    //INITIALIZE TILE PANEL
    public JPanel getTilePanel()
    {
        String[] images = new String[actualimages.length * 2];
        System.arraycopy(actualimages, 0, images, 0, actualimages.length);
        System.arraycopy(groundimages, 0, images, actualimages.length, groundimages.length);
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER, 5, 5));        
        JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 5, 5));
        for(int j = 0; j < images.length / 2; j++)
        {
            //create action listener
            MouseListener l = new MouseListener()
            {
                public void mousePressed(MouseEvent e)
                {
                    ImagePanel impanel = (ImagePanel)e.getSource();
                    if (firsttime == true) {
                        setImage2(impanel.getImageName());
                        firsttime = false;
                    }
                    else setImage(impanel.getImageName());
                    sqframe.setMode(2);
                }
                public void mouseEntered(MouseEvent e) {}
                public void mouseExited(MouseEvent e) {}
                public void mouseClicked(MouseEvent e) {}
                public void mouseReleased(MouseEvent e) {}
            };
            //create image panel
            BufferedImage tempimage = null;
            try {
                tempimage = ImageIO.read(new File(images[j]));
            } catch (Exception e) {
                e.printStackTrace();
            }
            int newW = tileW;
            int newH = tileH;
            BufferedImage tempimage2 = resizeImage(tempimage, newW, newH);
            ImagePanel impanel = new ImagePanel(tempimage2, images[j], newW, newH);
            impanel.addMouseListener(l);
            panel.add(impanel);
        }
        for(int j = images.length / 2; j < images.length; j++)
        {
            //create action listener
            MouseListener l = new MouseListener()
            {
                public void mousePressed(MouseEvent e)
                {
                    ImagePanel impanel = (ImagePanel)e.getSource();
                    if (firsttime == true) {
                        setImage2(impanel.getImageName());
                        firsttime = false;
                    }
                    else setImage(impanel.getImageName());
                    sqframe.setMode(2);
                }
                public void mouseEntered(MouseEvent e) {}
                public void mouseExited(MouseEvent e) {}
                public void mouseClicked(MouseEvent e) {}
                public void mouseReleased(MouseEvent e) {}
            };
            //create image panel
            BufferedImage tempimage = null;
            try {
                tempimage = ImageIO.read(new File(images[j]));
            } catch (Exception e) {
                e.printStackTrace();
            }
            int newW = tileW;
            int newH = tileH;
            BufferedImage tempimage2 = resizeImage(tempimage, newW, newH);
            ImagePanel impanel = new ImagePanel(tempimage2, images[j], newW, newH);
            impanel.addMouseListener(l);
            panel2.add(impanel);
        }
        JPanel panel3 = new JPanel();
        panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));
//         JLabel label1 = new JLabel("Original Images");
//         JLabel label2 = new JLabel("Foreground");
//         JLabel emptylabel = new JLabel(" ");
        //set font sizes
//         Font curFont = label1.getFont();
        int fontsize = 20;
//         label1.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
//         label2.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
//         emptylabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        //create panels for labels
//         JPanel labpanel1 = new JPanel(new FlowLayout(FlowLayout.CENTER, 5, 5));
//         JPanel labpanel2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 5, 5));
//         labpanel1.add(label1);
//         labpanel2.add(label2);
        //labpanel1.setPreferredSize(new Dimension(500,40));
        //add panels to final panel
//         panel3.add(emptylabel);
//         panel3.add(labpanel1);
        panel3.add(panel);
//         panel3.add(labpanel2);
//         panel3.add(panel2);
        return panel3;
    }
    
    //INITIALIZE CONTROL PANEL
    public JPanel getControlPanel()
    {
        //JLabel for current color
        if (color.equals(Color.red)) current_color.setText("Current scribble: Background");
        else current_color.setText("Current scribble: Foreground");
        MouseListener l = new MouseListener()
        {
            public void mousePressed(MouseEvent e)
            {
                ColorPanel p = (ColorPanel)e.getSource();
                color = p.getColor();
                swapColor(color);
                if (color.equals(Color.red)) current_color.setText("Current scribble: Background");
                else current_color.setText("Current scribble: Foreground");
            }
            public void mouseEntered(MouseEvent e) {}
            public void mouseExited(MouseEvent e) {}
            public void mouseClicked(MouseEvent e) {}
            public void mouseReleased(MouseEvent e) {}
        };
        //mode button
        JButton mode = new JButton("Mode");
        mode.setPreferredSize(new Dimension(buttonwidth, buttonheight));
        mode.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if (sqframe.getMode() == 1) sqframe.setMode(0);
                else sqframe.setMode(1);
            }
        });
        //undo button
        JButton undo = new JButton("Undo Scribble");
        undo.setPreferredSize(new Dimension(buttonwidth, buttonheight));
        undo.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if (squiggles_temp.size() > 0 && totalpixels < maxpixels) {
                    squiggles_temp.remove(squiggles_temp.size()-1);
                    squiggles.remove(squiggles.size()-1);
                    imagenames.remove(imagenames.size()-1);
                    clearImage();
                    showSquiggle();
                    refreshAllPixels();
                }
            }
        });
        //clear button
        JButton clear = new JButton("Hide Scribbles");
        clear.setPreferredSize(new Dimension(buttonwidth, buttonheight));
        clear.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                clearImage();
            }
        });
        //show button
        JButton show = new JButton("Show Scribbles");
        show.setPreferredSize(new Dimension(buttonwidth, buttonheight));
        show.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                showSquiggle();
            }
        });
        //segment button
        JButton segment = new JButton("Segment now!");
        segment.setPreferredSize(new Dimension(buttonwidth, buttonheight));
        segment.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                pixels_since_last = 0;
                refreshPixelCount();
                isDone = true;
            }
        });
        //next group button
        JButton next = new JButton("End!");
        next.setPreferredSize(new Dimension(buttonwidth, buttonheight));
        next.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                nextGroup = true;
            }
        });
        //initialize jpanel
        JPanel panel = new JPanel();
        //add color buttons
        //add red button
        panel.add(new JLabel("Background: "));
        backgroundp = new ColorPanel(Color.red);
        Border b = BorderFactory.createLineBorder(Color.red, 5);
        backgroundp.setPreferredSize(new Dimension(buttonwidth, buttonheight));
        if (color.equals(Color.red)) backgroundp.setBackground(Color.red);
        else backgroundp.setBackground(Color.white);
        backgroundp.setBorder(b);
        backgroundp.addMouseListener(l);
        panel.add(backgroundp);
        //add blue button
        panel.add(new JLabel("Foreground: "));
        foregroundp = new ColorPanel(Color.blue);
        b = BorderFactory.createLineBorder(Color.blue, 5);
        foregroundp.setPreferredSize(new Dimension(buttonwidth, buttonheight));
        if (color.equals(Color.blue)) foregroundp.setBackground(Color.blue);
        else foregroundp.setBackground(Color.white);
        foregroundp.setBorder(b);
        foregroundp.addMouseListener(l);
        panel.add(foregroundp);
        //add other buttons
        panel.add(mode);
        panel.add(undo);
        panel.add(clear);
        panel.add(show);
        panel.add(segment);
        panel.add(next);
        //create outermost jpanel
        JPanel outer = new JPanel();
        outer.setLayout(new BoxLayout(outer, BoxLayout.Y_AXIS));
        outer.add(panel);
        //create panel with pixels and score
        JPanel inner = new JPanel(new FlowLayout(FlowLayout.CENTER, 50, 50));
        //set font sizes
        Font curFont = current_color.getFont();
        int fontsize = 20;
        current_color.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        pixelleftlab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        iterationlab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        scorelab.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        statuslabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), fontsize));
        //add current color
        inner.add(current_color);
        //add pixel counts and score
        //inner.add(pixelleftlab);
        //inner.add(iterationlab);
        //inner.add(scorelab);
        inner.add(statuslabel);
        outer.add(inner);
        return outer;
    }
}



//CLASS FOR MOUSE LISTENER
class DrawingListener implements MouseInputListener
{
    Squiggle squiggle;
    Point start;
    
    public DrawingListener(Squiggle sq)
    {
        this.squiggle = sq;
    }
    
    public void mousePressed(MouseEvent e)
    {
        start = e.getPoint();
        if (squiggle.getTotalPixels() < squiggle.getMaxPixels() && squiggle.pixels_since_last < squiggle.maxpixels_per_itr) mousePos(start);
    }
    
    public void mouseDragged(MouseEvent e)
    {
        if (squiggle.getTotalPixels() < squiggle.getMaxPixels() && squiggle.pixels_since_last < squiggle.maxpixels_per_itr) {
            //Draw stuff
            Point p = e.getPoint();
            //get coordinates
            int x1 = (int)p.getX();
            int y1 = (int)p.getY();
            int x0 = (int)start.getX();
            int y0 = (int)start.getY();
            //check for longer side
            if (Math.abs(x1 - x0) > Math.abs(y1 - y0)) {
                //y = mx + b
                double slope = (double) (y1 - y0) / (x1 - x0);
                double b = y1 - slope * x1;
                //x is longer side, interpolate along this axis
                int initial = x0 + 1;
                if (x1 < x0) initial = x0 - 1;
                for (int i = initial; ; ) {
                    if (x1 > x0 && i > x1) break;
                    if (x1 < x0 && i < x1) break;
                    p = new Point(i, (int)(slope * i + b));
                    if (squiggle.getTotalPixels() < squiggle.getMaxPixels() && squiggle.pixels_since_last < squiggle.maxpixels_per_itr) mousePos(p);
                    else break;
                    if (squiggle.getBadSquiggle() == false) squiggle.draw(start, p);
                    start = new Point(p);
                    if (x1 > x0) i++;
                    else i--;
                }
            }
            else if (Math.abs(x1 - x0) < Math.abs(y1 - y0)) {
                //x = my + b
                double slope = (double) (x1 - x0) / (y1 - y0);
                double b = x1 - slope * y1;
                //y is longer side, interpolate along this axis
                int initial = y0 + 1;
                if (y1 < y0) initial = y0 - 1;
                for (int i = initial; ; ) {
                    if (y1 > y0 && i > y1) break;
                    if (y1 < y0 && i < y1) break;
                    p = new Point((int)(slope * i + b), i);
                    if (squiggle.getTotalPixels() < squiggle.getMaxPixels() && squiggle.pixels_since_last < squiggle.maxpixels_per_itr) mousePos(p);
                    else break;
                    if (squiggle.getBadSquiggle() == false) squiggle.draw(start, p);
                    start = new Point(p);
                    if (y1 > y0) i++;
                    else i--;
                }
            }
        }
    }
    
    public void mouseReleased(MouseEvent e) {
        mouseRel(e);
    }
    
    //mouse position
    private void mousePos(MouseEvent e) {
        Point point = e.getPoint();
        squiggle.addPoint((int)point.getX(), (int)point.getY());
        //System.out.println(point.toString());
    }
    
    private void mousePos(Point temp) {
        squiggle.addPoint((int)temp.getX(), (int)temp.getY());
        //System.out.println(temp.toString());
    }
    
    //mouse released
    private void mouseRel(MouseEvent e) {
        if (squiggle.getTotalPixels() < squiggle.getMaxPixels() && squiggle.pixels_since_last < squiggle.maxpixels_per_itr) squiggle.finishSquiggle();
        //System.out.println("Mouse Released!");
    }
    
    //rest of listeners
    public void mouseMoved(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    
    
}



//class for image panels
class ImagePanel extends JPanel{
    
    private BufferedImage image;
    String imagefile;
    Color col;
    
    public ImagePanel() {}
    
    public ImagePanel(Color c) {
        col = c;
    }        
    
    public ImagePanel(BufferedImage im, String s) {
        image = im;
        imagefile = s.toLowerCase();
        this.setPreferredSize(new Dimension(image.getWidth(), image.getHeight()));
    }
    
    public ImagePanel(BufferedImage im, String s, int panelW, int panelH) {
        image = im;
        imagefile = s.toLowerCase();
        this.setPreferredSize(new Dimension(panelW, panelH));
    }
    
    public String getImageName() {
        return imagefile;
    }
    
    public Color getColor() {
        return col;
    }
    
    @Override
    public void paintComponent(Graphics g) {
        try {
            g.drawImage(image, (getWidth()-image.getWidth())/2,
            (getHeight()-image.getHeight())/2, null); // see javadoc for more info on the parameters
        } catch (Exception e) {
        }
    }
    
}

//class for button panels
class ColorPanel extends JPanel{
    Color col;
    
    public ColorPanel() {}
    
    public ColorPanel(Color c) {
        col = c;
    }
    
    public Color getColor() {
        return col;
    }
    
}
